<?php

/**
 * Definitions for setting notification types
 */
/**
 * Utility definitions
 */
define('MDL_F2F_ICAL',			1);
define('MDL_F2F_TEXT',			2);
define('MDL_F2F_BOTH',          3);
define('MDL_F2F_INVITE',		4);
define('MDL_F2F_CANCEL',		8);

/**
 * Definitions for use in forms
 */
define('MDL_F2F_INVITE_BOTH',		7);	    // Send a copy of both 4+1+2
define('MDL_F2F_INVITE_TEXT',		6);	    // Send just a plain email 4+2
define('MDL_F2F_INVITE_ICAL',		5);	    // Send just a combined text/ical message 4+1
define('MDL_F2F_CANCEL_BOTH',		11);	// Send a copy of both 8+2+1
define('MDL_F2F_CANCEL_TEXT',		10);	// Send just a plan email 8+2
define('MDL_F2F_CANCEL_ICAL',		9);	    // Send just a combined text/ical message 8+1

/**
 * Prints the cost amount along with the appropriate currency symbol.
 *
 * To set your currency symbol, set the appropriate 'locale' in
 * lang/en_utf8/langconfig.php (or the equivalent file for your
 * language).
 *
 * @param $amount      Numerical amount without currency symbol
 * @param $htmloutput  Whether the output is in HTML or not
 */
function format_cost($amount, $htmloutput=true) {
    setlocale(LC_MONETARY, get_string('locale'));
    $localeinfo = localeconv();

    $symbol = $localeinfo['currency_symbol'];
    if (empty($symbol)) {
        // Cannot get the locale information, default to en_US.UTF-8
        return '$' . $amount;
    }

    // Character between the currency symbol and the amount
    $separator = '';
    if ($localeinfo['p_sep_by_space']) {
        $separator = $htmloutput ? '&nbsp;' : ' ';
    }

    // The symbol can come before or after the amount
    if ($localeinfo['p_cs_precedes']) {
        return $symbol . $separator . $amount;
    }
    else {
        return $amount . $separator . $symbol;
    }
}

/**
 * Returns the effective cost of a session depending on the presence
 * or absence of a discount code.
 *
 * @param class $sessiondata contains the discountcost and normalcost
 */
function facetoface_cost($userid, $sessionid, $sessiondata, $htmloutput=true) {

    global $CFG;

    if (count_records_sql("SELECT COUNT(*)
                               FROM {$CFG->prefix}facetoface_submissions su,
                                    {$CFG->prefix}facetoface_sessions se
                               WHERE su.sessionid=$sessionid AND su.userid=$userid AND
                                     su.discountcode IS NOT NULL AND su.sessionid = se.id") > 0) {
        return format_cost($sessiondata->discountcost, $htmloutput);
    } else {
        return format_cost($sessiondata->normalcost, $htmloutput);
    }
}

/**
 * Human-readable version of the duration field used to display it to
 * users
 *
 * @param integer $duration duration in minutes
 */
function facetoface_duration($duration) {

    $minutes = round(($duration - floor($duration)) * 60);
    $hours = floor($duration);

    $string = '';

    if (1 == $hours) {
        $string = get_string('onehour', 'facetoface');
    } elseif ($hours > 1) {
        $string = get_string('xhours', 'facetoface', $hours);
    }

    // Insert separator between hours and minutes
    if ($string != '') {
        $string .= ' ';
    }

    if (1 == $minutes) {
        $string .= get_string('oneminute', 'facetoface');
    } elseif ($minutes > 0) {
        $string .= get_string('xminutes', 'facetoface', $minutes);
    }

    return $string;
}

/**
 * Converts minutes to hours
 */
function facetoface_minutes_to_hours($minutes) {

    return round($minutes / 60.0, 2);
}

/**
 * Converts hours to minutes
 */
function facetoface_hours_to_minutes($hours) {

    return round($hours * 60.0);
}

/**
 * Turn undefined manager messages into empty strings
 */
function facetoface_fix_manager_messages($facetoface) {

    if (!$facetoface->emailmanagerconfirmation) {
        $facetoface->confirmationinstrmngr = '';
    }
    if (!$facetoface->emailmanagerreminder) {
        $facetoface->reminderinstrmngr = '';
    }
    if (!$facetoface->emailmanagercancellation) {
        $facetoface->cancellationinstrmngr = '';
    }
    if (!$facetoface->thirdpartywaitlist) {
        $facetoface->thirdpartywaitlist = 0;
    }
}

/**
 * Given an object containing all the necessary data, (defined by the
 * form in mod.html) this function will create a new instance and
 * return the id number of the new instance.
 */
function facetoface_add_instance($facetoface) {

    $facetoface->timemodified = time();

    facetoface_fix_manager_messages($facetoface);
    $returnid = insert_record('facetoface', $facetoface);

    return $returnid;

}

/**
 * Given an object containing all the necessary data, (defined by the
 * form in mod.html) this function will update an existing instance
 * with new data.
 */
function facetoface_update_instance($facetoface) {

    $facetoface->id = $facetoface->instance;

    facetoface_fix_manager_messages($facetoface);
    $returnid = update_record('facetoface', $facetoface);

    return $returnid;
}

/**
 * Given an ID of an instance of this module, this function will
 * permanently delete the instance and any data that depends on it.
 */
function facetoface_delete_instance($id) {

    global $CFG;

    if (!$facetoface = get_record('facetoface', 'id', $id)) {
        return false;
    }

    $result = true;
    begin_sql();

    if (!delete_records('facetoface_submissions', 'facetoface', $facetoface->id)) {
        $result = false;
    }

    if (!delete_records_select('facetoface_sessions_dates', "sessionid in (SELECT id FROM {$CFG->prefix}facetoface_sessions WHERE facetoface = $facetoface->id)")) {
        $result = false;
    }

    if (!delete_records('facetoface_sessions', 'facetoface', $facetoface->id)) {
        $result = false;
    }

    if (!delete_records('facetoface', 'id', $facetoface->id)) {
        $result = false;
    }

    if (!delete_records('event', 'modulename', 'facetoface', 'instance', $facetoface->id)) {
        $result = false;
    }

    if ($result) {
        commit_sql();
    } else {
        rollback_sql();
    }

    return $result;
}

/**
 * Prepare the user data to go into the database.
 */
function cleanup_session_data($session) {

    // Convert hours (expressed like "1.75" or "2" or "3.5") to minutes
    $session->duration = facetoface_hours_to_minutes($session->duration);

    // Only numbers allowed here
    $session->capacity = preg_replace('/[^\d]/', '', $session->capacity);
    $MAX_CAPACITY = 100000;
    if ($session->capacity < 1) {
        $session->capacity = 1;
    }
    elseif ($session->capacity > $MAX_CAPACITY) {
        $session->capacity = $MAX_CAPACITY;
    }

    // Get the decimal point separator
    setlocale(LC_MONETARY, get_string('locale'));
    $localeinfo = localeconv();
    $symbol = $localeinfo['decimal_point'];
    if (empty($symbol)) {
        // Cannot get the locale information, default to en_US.UTF-8
        $symbol = '.';
    }

    // Only numbers or decimal separators allowed here
    $session->normalcost = round(preg_replace("/[^\d$symbol]/", '', $session->normalcost));
    $session->discountcost = round(preg_replace("/[^\d$symbol]/", '', $session->discountcost));

    return $session;
}

/**
 * Create a new entry in the facetoface_sessions table
 */
function facetoface_add_session($session, $sessiondates) {

    $session->timecreated = time();
    $session = cleanup_session_data($session);

    begin_sql();
    if ($session->id = insert_record('facetoface_sessions', $session)) {
        foreach ($sessiondates as $date) {
            $date->sessionid = $session->id;
            if (!insert_record('facetoface_sessions_dates', $date)) {
                rollback_sql();
                return false;
            }
        }
        commit_sql();
        return $session->id;
    } else {
        rollback_sql();
        return false;
    }
}

/**
 * Modify an entry in the facetoface_sessions table
 */
function facetoface_update_session($session, $sessiondates) {

    $session->timemodified = time();
    $sessionid = $session->id;
    $session = cleanup_session_data($session);

    begin_sql();
    if (!$session->id = update_record('facetoface_sessions', $session)) {
        rollback_sql();
        return false;
    }

    if (!delete_records('facetoface_sessions_dates', 'sessionid', $sessionid)) {
        rollback_sql();
        return false;
    }
    foreach ($sessiondates as $date) {
        $date->sessionid = $sessionid;
        if (!insert_record('facetoface_sessions_dates', $date)) {
            rollback_sql();
            return false;
        }
    }

    commit_sql();
    return $session->id;
}

/**
 * Return an array of all facetoface activities in the current course
 */
function facetoface_get_facetoface_menu() {

    global $CFG;
	if ($facetofaces = get_records_sql("SELECT f.id, c.shortname, f.name
                                            FROM {$CFG->prefix}course c, {$CFG->prefix}facetoface f
                                            WHERE c.id = f.course
                                            ORDER BY c.shortname, f.name")) {
        $i=1;
        foreach ($facetofaces as $facetoface) {
            $f = $facetoface->id;
            $facetofacemenu[$f] = $facetoface->shortname.' --- '.$facetoface->name;
            $i++;
        }

        return $facetofacemenu;

    } else {
        
        return '';

    }
}

/**
 * Delete entry from the facetoface_sessions table along with all
 * related details in other tables
 */
function facetoface_delete_session($session) {

    begin_sql();
    if (!delete_records('facetoface_sessions', 'id', $session->s)) {
        rollback_sql();
        return false;
    }
    if (!delete_records('facetoface_submissions', 'sessionid', $session->s)) {
        rollback_sql();
        return false;
    }
    if (!delete_records('facetoface_sessions_dates', 'sessionid', $session->s)) {
        rollback_sql();
        return false;
    }

    commit_sql();
    return true;
}

/**
 * Subsitute the placeholders in email templates for the actual data
 */
function facetoface_email_substitutions($msg, $facetofacename, $reminderperiod, $user, $session, $sessionid) {

    $sessiondate = userdate($session->sessiondates[0]->timestart, get_string('strftimedate'));
    $starttime = userdate($session->sessiondates[0]->timestart, get_string('strftimetime'));
    $finishtime = userdate($session->sessiondates[0]->timefinish, get_string('strftimetime'));

    $alldates = '';
    foreach ($session->sessiondates as $date) {
        if ($alldates != '') {
            $alldates .= "\n";
        }
        $alldates .= userdate($date->timestart, get_string('strftimedate')).', ';
        $alldates .= userdate($date->timestart, get_string('strftimetime')).
            ' to '.userdate($date->timefinish, get_string('strftimetime'));
    }

    $msg = str_replace(get_string('placeholder:facetofacename', 'facetoface'), $facetofacename,$msg);
    $msg = str_replace(get_string('placeholder:firstname', 'facetoface'), $user->firstname,$msg);
    $msg = str_replace(get_string('placeholder:lastname', 'facetoface'), $user->lastname,$msg);
    $msg = str_replace(get_string('placeholder:cost', 'facetoface'), facetoface_cost($user->id, $sessionid, $session, false),$msg);
    $msg = str_replace(get_string('placeholder:alldates', 'facetoface'), $alldates,$msg);
    $msg = str_replace(get_string('placeholder:sessiondate', 'facetoface'), $sessiondate,$msg);
    $msg = str_replace(get_string('placeholder:starttime', 'facetoface'), $starttime,$msg);
    $msg = str_replace(get_string('placeholder:finishtime', 'facetoface'), $finishtime,$msg);
    $msg = str_replace(get_string('placeholder:duration', 'facetoface'), facetoface_duration($session->duration),$msg);
    $msg = str_replace(get_string('placeholder:location', 'facetoface'), $session->location,$msg);
    $msg = str_replace(get_string('placeholder:venue', 'facetoface'), $session->venue,$msg);
    $msg = str_replace(get_string('placeholder:room', 'facetoface'), $session->room,$msg);
    $msg = str_replace(get_string('placeholder:details', 'facetoface'), $session->details,$msg);
    $msg = str_replace(get_string('placeholder:reminderperiod', 'facetoface'), $reminderperiod,$msg);

    return $msg;
}

/**
 * Function to be run periodically according to the moodle cron
 * Finds all facetoface notifications that have yet to be mailed out, and mails them.
 */
function facetoface_cron () {

    global $CFG, $USER;

    if ($submissionsdata = facetoface_get_unmailed_reminders()) {

        $timenow   = time();

        foreach ($submissionsdata as $submissiondata) {

            if (facetoface_has_session_started($submissiondata, $timenow)) {
                // Too late, the session already started
                continue;
            }

            $reminderperiod = $submissiondata->reminderperiod;

            // Convert the period from business days (no weekends) to calendar days
            for ($reminderday = 0; $reminderday < $reminderperiod + 1; $reminderday++ ) {
                $reminderdaytime = $submissiondata->sessiondates[0]->timestart - ($reminderday * 24 * 3600);
                $reminderdaycheck = userdate($reminderdaytime, '%u');
                if ($reminderdaycheck > 5) {
                    // Saturdays and Sundays are not included in the
                    // reminder period as entered by the user, extend
                    // that period by 1
                    $reminderperiod++;
                }
            }

            $remindertime = $submissiondata->sessiondates[0]->timestart - ($reminderperiod * 24 * 3600);

            if ($timenow < $remindertime) {
                // Too early to send reminder
                continue;
            }

            if (! $user = get_record('user', 'id', "$submissiondata->userid")) {
                continue;
            }

            $USER->lang = $user->lang;

            if (! $course = get_record('course', 'id', "$submissiondata->course")) {
                continue;
            }

            if (! $facetoface = get_record('facetoface', 'id', "$submissiondata->facetofaceid")) {
                continue;
            }

            $postsubject = '';
            $posttext = '';
            $posttextmgrheading = '';

            if (empty ($submissiondata->mailedreminder)) {
                $postsubject = $facetoface->remindersubject;
                $posttext = $facetoface->remindermessage;
                $posttextmgrheading = $facetoface->reminderinstrmngr;
            }

            if (! empty($postsubject) && ! empty ($posttext) ) {

                $postsubject = facetoface_email_substitutions($postsubject, $submissiondata->facetofacename, $submissiondata->reminderperiod, $user, $submissiondata, $submissiondata->sessionid);
                $posttext = facetoface_email_substitutions($posttext, $submissiondata->facetofacename, $submissiondata->reminderperiod, $user, $submissiondata, $submissiondata->sessionid);

                if (! empty($posttextmgrheading) ) {
                    $posttextmgrheading = facetoface_email_substitutions($posttextmgrheading, $submissiondata->facetofacename, $submissiondata->reminderperiod, $user, $submissiondata, $submissiondata->sessionid);
                }

                $posthtml = '';
                $fromaddress = get_config(NULL, 'facetoface_fromaddress');
                if (!$fromaddress) {
                    $fromaddress = '';
                }

                require_once($CFG->dirroot.'/lib/adminlib.php');

                if (email_to_user($user, $fromaddress, $postsubject, $posttext, $posthtml)) {

                    echo get_string('sentreminderuser', 'facetoface').": $user->firstname $user->lastname $user->email<BR />\n";

                    $submission = new object;
                    $submission->id = $submissiondata->id;
                    $submission->mailedreminder = $timenow;
                    update_record('facetoface_submissions', $submission);

                    if (!empty($posttextmgrheading)) {
                        $managertext = $posttextmgrheading.$posttext;

                        $usercheck = get_record('user', 'id', $user->id);
                        $manager = $user;
                        $manager->email = $usercheck->msn;

                        if (!empty($manager->email) && email_to_user($manager, $fromaddress, $postsubject, $managertext, $posthtml)) {

                            echo get_string('sentremindermanager', 'facetoface').": $user->firstname $user->lastname $manager->email<BR />\n";

                        } else {
                            $errormsg = array();
                            $errormsg['submissionid'] = $submissiondata->id;
                            $errormsg['userid'] = $user->id;
                            $errormsg['manageremail'] = $usercheck->msn;
                            echo get_string('error:cronprefix', 'facetoface').' '.get_string('error:cannotemailmanager', 'facetoface', $errormsg)."\n";
                        }
                    }

                } else {
                    $errormsg = array();
                    $errormsg['submissionid'] = $submissiondata->id;
                    $errormsg['userid'] = $user->id;
                    $errormsg['useremail'] = $user->email;
                    echo get_string('error:cronprefix', 'facetoface').' '.get_string('error:cannotemailuser', 'facetoface', $errormsg)."\n";
                }
            }
        }
    } else {
        echo get_string('noremindersneedtobesent', 'facetoface');
    }
    return true;
}

/**
 * Returns true if the session has started, that is if one of the
 * session dates is in the past.
 *
 * @param class $session record from the facetoface_sessions table
 * @param integer $timenow current time
 */
function facetoface_has_session_started($session, $timenow) {

    if (!$session->datetimeknown) {
        return false; // no date set
    }

    foreach ($session->sessiondates as $date) {
        if ($date->timestart < $timenow) {
            return true;
        }
    }
    return false;
}

/**
 * Print the list of all sessions for the given facetoface activity and location
 */
function facetoface_print_sessions($courseid, $facetofaceid, $location) {

    global $CFG, $USER;

    $context = get_context_instance(CONTEXT_COURSE, $courseid, $USER->id);

    $bookedsession = '0';
    $spanclass = '';

    $submission = get_record_select('facetoface_submissions', "facetoface=$facetofaceid AND userid=$USER->id");

    $bookedsession = null;
    if ($submission) {
        $bookedsession = $submission->sessionid;
    }

    $timenow = time();

    $tableheader = '';

    if (has_capability('mod/facetoface:viewattendees', $context)) {
        $tableheader = '<thead>'
                    . '<tr>'
                    . '<th class="header" align="left">'.get_string('location', 'facetoface').'</th>'
                    . '<th class="header" align="left">'.get_string('venue', 'facetoface').'</th>'
                    . '<th class="header">'.get_string('date', 'facetoface').'</th>'
                    . '<th class="header">'.get_string('time', 'facetoface').'</th>'
                    . '<th class="header">'.get_string('capacity', 'facetoface').'</th>'
                    . '<th class="header">'.get_string('status', 'facetoface').'</th>'
                    . '<th class="header">'.get_string('options', 'facetoface').'</th>'
                    . '</tr>'
                    . '</thead>';
    } else {
        $tableheader = '<thead>'
                    . '<tr>'
                    . '<th class="header" align="left">'.get_string('location', 'facetoface').'</th>'
                    . '<th class="header" align="left">'.get_string('venue', 'facetoface').'</th>'
                    . '<th class="header">'.get_string('date', 'facetoface').'</th>'
                    . '<th class="header">'.get_string('time', 'facetoface').'</th>'
                    . '<th class="header">'.get_string('seatsavailable', 'facetoface').'</th>'
                    . '<th class="header">'.get_string('status', 'facetoface').'</th>'
                    . '<th class="header">'.get_string('options', 'facetoface').'</th>'
                    . '</tr>'
                    . '</thead>';
    }

    $tableupcoming = '';
    $tableupcomingtbd = '';
    $tableprevious = '';

    if ($sessions = facetoface_get_sessions($facetofaceid, $location) ) {

        foreach($sessions as $session) {

            $status  = get_string('bookingopen', 'facetoface');
            $options = '';
            $spanclass = '';

            $signupcount = count_records('facetoface_submissions', 'sessionid', $session->id);

            if ($signupcount >= $session->capacity) {

                $status = get_string('bookingfull', 'facetoface');

            } elseif ($session->closed) {

                $status = get_string('bookingclosed', 'facetoface');

            }

            $allsessiondates = '';
            $allsessiontimes = '';
            foreach ($session->sessiondates as $date) {
                if (!empty($allsessiondates)) {
                        $allsessiondates .= '<br />';
                }
                $allsessiondates .= userdate($date->timestart, get_string('strftimedate'));
                if (!empty($allsessiontimes)) {
                    $allsessiontimes .= '<br />';
                }
                $allsessiontimes .= userdate($date->timestart, get_string('strftimetime')).
                    ' - '.userdate($date->timefinish, get_string('strftimetime'));
            }

            // Defaults for normal users
            $stats = $session->capacity - $signupcount;
            $options = '';

            if ($session->datetimeknown && facetoface_has_session_started($session, $timenow)) {

                $status = get_string('sessionover', 'facetoface');
                $spanclass = ' class="dimmed_text"';

                if (has_capability('mod/facetoface:editsessions', $context)) {
                    $options .= ' <a href="sessions.php?s='.$session->id.'" title="'.get_string('editsession', 'facetoface').'">'.get_string('edit').'</a> '
                        . '<a href="sessions.php?s='.$session->id.'&amp;c=1" title="'.get_string('copysession', 'facetoface').'">'.get_string('copy', 'facetoface').'</a> '
                        . '<a href="sessions.php?s='.$session->id.'&amp;d=1" title="'.get_string('deletesession', 'facetoface').'">'.get_string('delete').'</a> ';
                }
                if (has_capability('mod/facetoface:viewattendees', $context)){
                    $stats = $signupcount.' / '.$session->capacity;
                    $options .= '<a href="attendees.php?s='.$session->id.'" title="'.get_string('seeattendees', 'facetoface').'">'.get_string('attendees', 'facetoface').'</a> ';
                }

                if (empty($options)) {
                    $options = get_string('none', 'facetoface');
                }

                $tableprevious .= '<tr>'
                            . '<td class="content" style="width: 10%"><span '.$spanclass.'>'.$session->location.'</span></td>'
                            . '<td class="content" style="width: 20%"><span '.$spanclass.'>'.$session->venue.'</span></td>'
                            . '<td class="content" style="width: 10%" align="center"><span '.$spanclass.'>'.$allsessiondates.'</span></td>'
                            . '<td class="content" style="width: 15%" align="center"><span '.$spanclass.'>'.$allsessiontimes.'</span></td>'
                            . '<td class="content" style="width: 15%" align="center"><span '.$spanclass.'>'.$stats.'</span></td>'
                            . '<td class="content" style="width: 10%" align="center"><span '.$spanclass.'>'.$status.'</span></td>'
                            . '<td class="content" style="width: 20%" align="center"><span '.$spanclass.'>'.$options.'</span></td>'
                            . '</tr>';

            } else {

                $trclass = '';
                $spanclass = '';

                if (has_capability('mod/facetoface:editsessions', $context)) {
                    $options .= '<a href="sessions.php?s='.$session->id.'" title="'.get_string('editsession', 'facetoface').'">'.get_string('edit').'</a> '
                        . '<a href="sessions.php?s='.$session->id.'&amp;c=1" title="'.get_string('copysession', 'facetoface').'">'.get_string('copy', 'facetoface').'</a> '
                        . '<a href="sessions.php?s='.$session->id.'&amp;d=1" title="'.get_string('deletesession', 'facetoface').'">'.get_string('delete').'</a> ';
                }
                if (has_capability('mod/facetoface:viewattendees', $context)) {
                    $stats = $signupcount.' / '.$session->capacity;
                    $options .= ' <a href="attendees.php?s='.$session->id.'" title="'.get_string('seeattendees', 'facetoface').'">'.get_string('attendees', 'facetoface').'</a> ';
                }
                if ($session->id == $bookedsession) {
                    $trclass = ' class="highlight"';
                    $tableupcoming .= '<tr'.$trclass.'><td class="content" colspan="7"><span style="font-size: 12px; line-height: 12px;"><b>'.get_string('youarebooked', 'facetoface').':</b></span></td></tr>';

                    $options .= '<a href="'.$CFG->wwwroot.'/mod/facetoface/signup.php?s='.$session->id.'&amp;viewall='.$facetofaceid.'" alt="'.get_string('moreinfo', 'facetoface').'" title="'.get_string('moreinfo', 'facetoface').'">'.get_string('moreinfo', 'facetoface').'</a><br />'
                        . '<a href="'.$CFG->wwwroot.'/mod/facetoface/attendees.php?s='.$session->id.'&amp;viewall='.$facetofaceid.'" alt="'.get_string('seeattendees', 'facetoface').'" title="'.get_string('seeattendees', 'facetoface').'">'.get_string('seeattendees', 'facetoface').'</a><br />'
                        . '<a href="'.$CFG->wwwroot.'/mod/facetoface/signup.php?s='.$session->id.'&amp;cancelbooking=1&amp;viewall='.$facetofaceid.'" alt="'.get_string('cancelbooking', 'facetoface').'" title="'.get_string('cancelbooking', 'facetoface').'">'.get_string('cancelbooking', 'facetoface').'</a>';
                } else {
                    if ($bookedsession || ($status == get_string('bookingfull', 'facetoface'))) {
                        $spanclass = ' class="dimmed_text"';
                    } else {
                        $options .= '<a href="signup.php?s='.$session->id.'&amp;viewall='.$facetofaceid.'">'.get_string('signuplowercase', 'facetoface').'</a>';
                    }
                }

                if (empty($options)) {
                    $options = get_string('none', 'facetoface');
                }

                if ($session->datetimeknown) {

                    $tableupcoming .= '<tr'.$trclass.'>'
                                . '<td class="content" style="width: 10%"><span '.$spanclass.'>'.$session->location.'</span></td>'
                                . '<td class="content" style="width: 20%"><span '.$spanclass.'>'.$session->venue.'</span></td>'
                                . '<td class="content" style="width: 10%" align="center"><span '.$spanclass.'>'.$allsessiondates.'</span></td>'
                                . '<td class="content" style="width: 15%" align="center"><span '.$spanclass.'>'.$allsessiontimes.'</span></td>'
                                . '<td class="content" style="width: 15%" align="center"><span '.$spanclass.'>'.$stats.'</span></td>'
                                . '<td class="content" style="width: 10%" align="center"><span '.$spanclass.'>'.$status.'</span></td>'
                                . '<td class="content" style="width: 20%" align="center"><span '.$spanclass.'>'.$options.'</span></td>'
                                . '</tr>';

                } else {

                    $tableupcomingtbd .= '<tr'.$trclass.'>'
                                . '<td class="content" style="width: 10%"><span '.$spanclass.'>'.$session->location.'</span></td>'
                                . '<td class="content" style="width: 20%"><span '.$spanclass.'>'.$session->venue.'</span></td>'
                                . '<td class="content" style="width: 10%" align="center"><span '.$spanclass.'>'.get_string('wait-listed', 'facetoface').'</span></td>'
                                . '<td class="content" style="width: 15%" align="center"><span '.$spanclass.'>'.get_string('wait-listed', 'facetoface').'</span></td>'
                                . '<td class="content" style="width: 15%" align="center"><span '.$spanclass.'>'.$stats.'</span></td>'
                                . '<td class="content" style="width: 10%" align="center"><span '.$spanclass.'>'.get_string('wait-listed', 'facetoface').'</span></td>'
                                . '<td class="content" style="width: 20%" align="center"><span '.$spanclass.'>'.$options.'</span></td>'
                                . '</tr>';

                }

            }

        }

    }

    print_heading(get_string('upcomingsessions', 'facetoface'), 'center');
        echo '<table align="center" cellpadding="3" cellspacing="0" width="90%" style="border-color:#DDDDDD; border-width:1px 1px 1px 1px; border-style:solid;">';
        echo $tableheader;
        if (empty($tableupcoming) and empty($tableupcomingtbd) ) {
            echo '<tr><td colspan="7" align="center">'.get_string('noupcoming', 'facetoface').'</td></tr>';
        } else {
            echo $tableupcoming;
            echo $tableupcomingtbd;
        }
        if (has_capability('mod/facetoface:editsessions', $context)) {
            echo '<tr><td colspan="7" align="center"><a href="'.$CFG->wwwroot.'/mod/facetoface/sessions.php?f='.$facetofaceid.'" title="'.get_string('addsession', 'facetoface').'">'.get_string('addsession', 'facetoface').'</a></td></tr>';
        }
        echo '</table>';

    if (! empty($tableprevious)) {
        print_heading(get_string('previoussessions', 'facetoface'), 'center');
        echo '<table align="center" cellpadding="3" cellspacing="0" width="90%" style="border-color:#DDDDDD; border-width:1px 1px 1px 1px; border-style:solid;">';
        echo $tableheader;
        echo $tableprevious;
        echo '</table>';
    } 

}

/**
 * Get all of the dates for a given session
 */
function facetoface_get_session_dates($sessionid) {

    $ret = array();

    if ($dates = get_records('facetoface_sessions_dates', 'sessionid', $sessionid)) {
        $i = 0;
        foreach ($dates as $date) {
            $ret[$i++] = $date;
        }
    }

    return $ret;
}

/**
 * Get a record from the facetoface_sessions table
 *
 * @param integer $sessionid ID of the session
 */
function facetoface_get_session($sessionid) {

    $session = get_record('facetoface_sessions', 'id', $sessionid);

    if ($session) {
        $session->sessiondates = facetoface_get_session_dates($sessionid);
        $session->duration = facetoface_minutes_to_hours($session->duration);
    }

    return $session;
}

/**
 * Get all records from facetoface_sessions for a given facetoface activity and location
 *
 * @param integer $facetofaceid ID of the activity
 * @param string $location location filter (optional)
 */
function facetoface_get_sessions($facetofaceid, $location='') {

    global $CFG;
    if (empty($location)) {
        $sessions = get_records_sql("SELECT s.* FROM {$CFG->prefix}facetoface_sessions s,
                                        (SELECT sessionid, min(timestart) AS mintimestart
                                            FROM {$CFG->prefix}facetoface_sessions_dates GROUP BY sessionid) d
                                        WHERE s.facetoface=$facetofaceid AND d.sessionid = s.id
                                        ORDER BY s.datetimeknown, d.mintimestart");

        $brokensessions = get_records_sql("SELECT s.* FROM {$CFG->prefix}facetoface_sessions s
                                               WHERE s.facetoface=$facetofaceid
                                                   AND NOT EXISTS
                                          (SELECT 1 FROM {$CFG->prefix}facetoface_sessions_dates
                                              WHERE sessionid = s.id)");
    } else {
        $sessions = get_records_sql("SELECT s.* FROM {$CFG->prefix}facetoface_sessions s,
                                        (SELECT sessionid, min(timestart) AS mintimestart
                                            FROM {$CFG->prefix}facetoface_sessions_dates GROUP BY sessionid) d
                                        WHERE s.facetoface=$facetofaceid AND d.sessionid = s.id
                                            AND s.location='$location'
                                        ORDER BY s.datetimeknown, d.mintimestart");

        $brokensessions = get_records_sql("SELECT s.* FROM {$CFG->prefix}facetoface_sessions s
                                               WHERE s.facetoface=$facetofaceid
                                                   AND s.location='$location'
                                                   AND NOT EXISTS
                                          (SELECT 1 FROM {$CFG->prefix}facetoface_sessions_dates
                                              WHERE sessionid = s.id)");
    }

    // Broken sessions are sessions which have no dates associated
    // with them, they are only returned so that they are visible and
    // can be fixed by users.  The cause of these broken sessions
    // should be investigated and a bug should be filed.
    if ($brokensessions) {
        $courseid = get_field('facetoface', 'course', 'id', $facetofaceid);
        add_to_log($courseid, 'facetoface', get_string('log:brokensessions', 'facetoface'), '', "facetofaceid=$facetofaceid");
        $sessions = array_merge($sessions, $brokensessions);
    }

    if ($sessions) {
        foreach ($sessions as $key => $value) {
            $sessions[$key]->duration = facetoface_minutes_to_hours($sessions[$key]->duration);
            $sessions[$key]->sessiondates = facetoface_get_session_dates($value->id);
        }
    }

    return $sessions;
}

/**
 * Get list of users attending a given session
 */
function facetoface_get_attendees($sessionid) {
    global $CFG;
    return get_records_sql("SELECT s.id AS submissionid, u.id, u.firstname, u.lastname, s.grade, s.discountcode
                                FROM {$CFG->prefix}facetoface_submissions s, {$CFG->prefix}user u
                                WHERE s.sessionid=$sessionid AND u.id=s.userid
                                ORDER BY u.firstname");
}

/**
 * Download the list of users attending at least one of the sessions
 * for a given facetoface activity
 */
function facetoface_download_attendance($facetofacename, $facetofaceid, $location, $format) {
    global $CFG;

    $timenow = time();
    $timeformat = str_replace(' ', '_', get_string('strftimedate'));
    $downloadfilename = clean_filename($facetofacename.'_'.userdate($timenow, $timeformat));

    if ('ods' === $format) {
        // OpenDocument format (ISO/IEC 26300)
        require_once($CFG->dirroot.'/lib/odslib.class.php');
        $downloadfilename .= '.ods';
        $workbook = new MoodleODSWorkbook('-');
    }
    else {
        // Excel format
        require_once($CFG->dirroot.'/lib/excellib.class.php');
        $downloadfilename .= '.xls';
        $workbook = new MoodleExcelWorkbook('-');
    }

    $workbook->send($downloadfilename);
    $worksheet =& $workbook->add_worksheet('attendance');
    $worksheet->write_string(0,0,get_string('location', 'facetoface'));
    $worksheet->write_string(0,1,get_string('venue', 'facetoface'));
    $worksheet->write_string(0,2,get_string('date', 'facetoface'));
    $worksheet->write_string(0,3,get_string('timestart', 'facetoface'));
    $worksheet->write_string(0,4,get_string('timefinish', 'facetoface'));
    $worksheet->write_string(0,5,get_string('status', 'facetoface'));
    $worksheet->write_string(0,6,get_string('firstname'));
    $worksheet->write_string(0,7,get_string('lastname'));
    $worksheet->write_string(0,8,get_string('email'));
    $worksheet->write_string(0,9,get_string('manageremail', 'facetoface'));
    $worksheet->write_string(0,10,get_string('city'));
    $worksheet->write_string(0,11,get_string('idnumber'));
    $worksheet->write_string(0,12,get_string('institution'));
    $worksheet->write_string(0,13,get_string('department'));
    $worksheet->write_string(0,14,get_string('address'));
    $worksheet->write_string(0,15,get_string('attendance', 'facetoface'));

    $i = 0;

    $sessions = facetoface_get_sessions($facetofaceid, $location);

    if (! empty ($sessions) ) {

        foreach ($sessions as $session) {

            if ($session->datetimeknown) {

                // Display only the first date
                $sessiondate = userdate($session->sessiondates[0]->timestart, get_string('strftimedate'));
                $starttime   = userdate($session->sessiondates[0]->timestart, get_string('strftimetime'));
                $finishtime  = userdate($session->sessiondates[0]->timefinish, get_string('strftimetime'));

                if (facetoface_has_session_started($session, $timenow)) {

                    $status = get_string('sessionover', 'facetoface');

                } else {

                    $signupcount = count_records('facetoface_submissions', 'sessionid', $session->id);

                    if ($signupcount >= $session->capacity) {

                        $status = get_string('bookingfull', 'facetoface');

                    } else {

                        $status = get_string('bookingopen', 'facetoface');

                    }

                }

            } else {

                $sessiondate = get_string('wait-listed', 'facetoface');
                $starttime   = get_string('wait-listed', 'facetoface');
                $finishtime  = get_string('wait-listed', 'facetoface');
                $status      = get_string('wait-listed', 'facetoface');

            }

            $attendees = facetoface_get_attendees($session->id);

            if (! empty ($attendees) ) {

                foreach ($attendees as $attendee) {

                    $student = get_complete_user_data('id', $attendee->id);

                    if (! empty ($student) ) {

                        $i++;

                        $worksheet->write_string($i,0,$session->location);
                        $worksheet->write_string($i,1,$session->venue);
                        $worksheet->write_string($i,2,$sessiondate);
                        $worksheet->write_string($i,3,$starttime);
                        $worksheet->write_string($i,4,$finishtime);
                        $worksheet->write_string($i,5,$status);
                        $worksheet->write_string($i,6,$student->firstname);
                        $worksheet->write_string($i,7,$student->lastname);
                        $worksheet->write_string($i,8,$student->email);
                        $worksheet->write_string($i,9,$student->msn);
                        $worksheet->write_string($i,10,$student->city);
                        $worksheet->write_string($i,11,$student->idnumber);
                        $worksheet->write_string($i,12,$student->institution);
                        $worksheet->write_string($i,13,$student->department);
                        $worksheet->write_string($i,14,$student->address);
                        $worksheet->write_string($i,15,$attendee->grade);

                    }

                }

            } else {
                // no one is sign-up, so let's just print the basic info

                $i++;

                $worksheet->write_string($i,0,$session->location);
                $worksheet->write_string($i,1,$session->venue);
                $worksheet->write_string($i,2,$sessiondate);
                $worksheet->write_string($i,3,$starttime);
                $worksheet->write_string($i,4,$finishtime);
                $worksheet->write_string($i,5,$status);
                $worksheet->write_string($i,6,'-');
                $worksheet->write_string($i,7,'-');
                $worksheet->write_string($i,8,'-');
                $worksheet->write_string($i,9,'-');
                $worksheet->write_string($i,10,'-');
                $worksheet->write_string($i,11,'-');
                $worksheet->write_string($i,12,'-');
                $worksheet->write_string($i,13,'-');
                $worksheet->write_string($i,14,'-');
                $worksheet->write_string($i,15,'-');

            }

        }

    }

    $workbook->close();
    exit;

}

/**
 * Must return an array of grades, indexed by user, and a max grade
 */
function facetoface_grades($facetofaceid) {

    global $course;

    if ($grades = get_records_menu('facetoface_submissions', 'facetoface', $facetofaceid, '', 'userid, grade')) {

            $return->grades = $grades;
            $return->maxgrade = '';
            return $return;

    } else {
        return NULL;
    }
}

/**
 * Return an array of all of the locations where the given facetoface
 * activity has sessions
 */
function facetoface_get_locations($facetofaceid) { 
    global $CFG; 
    if ($sessions = get_records_sql("SELECT DISTINCT location, id, venue
                                         FROM {$CFG->prefix}facetoface_sessions
                                         WHERE facetoface = $facetofaceid
                                         ORDER BY location")) {

        $i=1;
        $locationmenu[''] = get_string('alllocations', 'facetoface');
        foreach ($sessions as $session) {
            $f = $session->id;
            $locationmenu[$session->location] = $session->location;
            $i++;
        }

        return $locationmenu;

    } else {
        
        return '';

    }
}

/**
 * Return list of marked submissions that have not been mailed out for currently enrolled students
 */
function facetoface_get_unmailed_reminders() {

    global $CFG;

    $submissions = get_records_sql("SELECT su.*, f.course, f.id as facetofaceid, f.name as facetofacename,
                                           f.reminderperiod, se.duration, se.normalcost, se.discountcost,
                                           se.location, se.venue, se.room, se.details, se.datetimeknown
                                       FROM {$CFG->prefix}facetoface_submissions su,
                                            {$CFG->prefix}facetoface_sessions se,
                                            {$CFG->prefix}facetoface f,
                                            {$CFG->prefix}course c
                                       WHERE su.mailedreminder = 0 AND se.datetimeknown=1 AND
                                             f.course=c.id AND su.sessionid=se.id AND
                                             se.facetoface=f.id AND f.id=su.facetoface");

    foreach ($submissions as $key => $value) {
        $submissions[$key]->duration = facetoface_minutes_to_hours($submissions[$key]->duration);
        $submissions[$key]->sessiondates = facetoface_get_session_dates($value->sessionid);
    }

    return $submissions;
}

/**
 * Return the record from the facetoface_submissions table for the
 * given facetoface activity and user
 *
 * @param class $facetoface record from the facetoface table
 */
function facetoface_get_submission($facetoface, $user) {
    $submission = get_record('facetoface_submissions', 'facetoface', $facetoface->id, 'userid', $user->id);
    if (!empty($submission->timemodified)) {
        return $submission;
    }
    return NULL;
}

/**
 * Add a record to the facetoface submissions table and sends out an
 * email confirmation
 *
 * @param class $session record from the facetoface_sessions table
 * @param class $facetoface record from the facetoface table
 * @param class $course record from the course table
 * @param string $discountcode code entered by the user
 * @param integer $notificationtype type of notifications to send to user
 * @see {{MDL_F2F_INVITE}}
 * @param integer $userid user to signup
 * @param bool $notifyuser whether or not to send an email confirmation
 * @param bool $displayerrors whether or not to return an error page on errors
 */
function facetoface_user_signup($session, $facetoface, $course, $discountcode,
                                $notificationtype, $userid=false,
                                $notifyuser=true, $displayerrors=true) {
    global $USER;

    if (!$userid) {
        $userid = $USER->id;
    }

    $return = false;

    $usersignup = new stdclass;
    $usersignup->sessionid = $session->id;
    $usersignup->userid = $userid;
    $usersignup->facetoface = $session->facetoface;
    $usersignup->timecreated = time();
    $usersignup->discountcode = trim(strtoupper($discountcode));
    if (empty($usersignup->discountcode)) {
        $usersignup->discountcode = null;
    }

    $usersignup->notificationtype = $notificationtype;

    begin_sql();
    if ($returnid = insert_record('facetoface_submissions', $usersignup)) {

        $return = $returnid;
        $timenow = time();

        if (!$notifyuser or facetoface_has_session_started($session, $timenow)) {
            // If the session has already started, there's no need to notify the user
            commit_sql();
            return $return;
        }
        else {
            $error = facetoface_send_confirmation_notice($facetoface, $session, $userid, $notificationtype);
            if (empty($error)) {
                $usersignup->id = $returnid;
                $usersignup->mailedconfirmation = $timenow;

                if (update_record('facetoface_submissions', $usersignup)) {
                    commit_sql();
                    return $return;
                }
            }
            elseif ($displayerrors) {
                error($error);
            }
        }
    }

    rollback_sql();
    return false;
}

/**
 * Cancel a user who signed up earlier
 *
 * @param class $session record from the facetoface_sessions table
 * @param integer $userid ID of the user to remove from the session
 */
function facetoface_user_cancel($session, $userid=false) {

    if (!$userid) {
        global $USER;
        $userid = $USER->id;
    }

    begin_sql();
    if ($userid && $session->id &&
        delete_records('facetoface_submissions', 'sessionid', $session->id, 'userid', $userid)) {

        commit_sql();
        return true;

    } else {
        rollback_sql();
        return false;
    }
}

/**
 * Common code for sending confirmation and cancellation notices
 *
 * @param string $postsubject Subject of the email
 * @param string $posttext Plain text contents of the email
 * @param string $posttextmgrheading Header to prepend to $posttext in manager email
 * @param string $notificationtype The type of notification to send
 * @see {{MDL_F2F_INVITE}}
 * @param class $facetoface record from the facetoface table
 * @param class $session record from the facetoface_sessions table
 * @param integer $userid ID of the recipient of the email
 * @returns string Error message (or empty string if successful)
 */
function facetoface_send_notice($postsubject, $posttext, $posttextmgrheading,
                                $notificationtype, $facetoface, $session, $userid) {
    global $CFG;

    $user = get_record('user', 'id', $userid);
    if (!$user) {
        return get_string('error:invaliduserid', 'facetoface');
    }

    if (empty($postsubject) || empty($posttext)) {
        return '';
    }

    // If no notice type is defined (TEXT or ICAL)
    if (!($notificationtype & MDL_F2F_BOTH)) {
        // If none, make sure they at least get a text email
        $notificationtype |= MDL_F2F_TEXT;
    }

    // If we are cancelling, check if ical cancellations are disabled
    if (($notificationtype & MDL_F2F_CANCEL) &&
        get_config(NULL, 'facetoface_disableicalcancel')) {
        $notificationtype |= MDL_F2F_TEXT; // add a text notification
        $notificationtype &= ~MDL_F2F_ICAL; // remove the iCalendar notification
    }

    // If we are sending an ical attachment, set file name
    if ($notificationtype & MDL_F2F_ICAL) {
        if ($notificationtype & MDL_F2F_INVITE) {
            $attachmentfilename = 'invite.ics';
        }
	    elseif ($notificationtype & MDL_F2F_CANCEL) {
	        $attachmentfilename = 'cancel.ics';
	    }
    }

    // Do iCal attachement stuff
    $icalattachments = array();
    if ($notificationtype & MDL_F2F_ICAL) {
        if (get_config(NULL, 'facetoface_oneemailperday')) {
            // Keep track of all sessiondates
            $sessiondates = $session->sessiondates;

            foreach ($sessiondates as $sessiondate) {
                $session->sessiondates = array($sessiondate); // one day at a time

                $filename = facetoface_get_ical_attachment($notificationtype, $facetoface, $session, $user);
                $subject = facetoface_email_substitutions($postsubject, $facetoface->name, $facetoface->reminderperiod,
                                                          $user, $session, $session->id);
                $body = facetoface_email_substitutions($posttext, $facetoface->name, $facetoface->reminderperiod,
                                                       $user, $session, $session->id);
                $htmlbody = ''; // TODO
                $icalattachments[] = array('filename' => $filename, 'subject' => $subject,
                                           'body' => $body, 'htmlbody' => $htmlbody);
            }

            // Restore session dates
            $session->sessiondates = $sessiondates;
        }
        else {
            $filename = facetoface_get_ical_attachment($notificationtype, $facetoface, $session, $user);
            $subject = facetoface_email_substitutions($postsubject, $facetoface->name, $facetoface->reminderperiod,
                                                      $user, $session, $session->id);
            $body = facetoface_email_substitutions($posttext, $facetoface->name, $facetoface->reminderperiod,
                                                   $user, $session, $session->id);
            $htmlbody = ''; // TODO
            $icalattachments[] = array('filename' => $filename, 'subject' => $subject,
                                       'body' => $body, 'htmlbody' => $htmlbody);
        }
    }

    // Fill-in the email placeholders
    $postsubject = facetoface_email_substitutions($postsubject, $facetoface->name, $facetoface->reminderperiod,
                                                  $user, $session, $session->id);
    $posttext = facetoface_email_substitutions($posttext, $facetoface->name, $facetoface->reminderperiod,
                                               $user, $session, $session->id);

    if (!empty($posttextmgrheading)) {
        $posttextmgrheading = facetoface_email_substitutions($posttextmgrheading, $facetoface->name,
                                                             $facetoface->reminderperiod, $user, $session,
                                                             $session->id);
    }

    $posthtml = ''; // TODO: provide an HTML version of these notices
    $fromaddress = get_config(NULL, 'facetoface_fromaddress');
    if (!$fromaddress) {
        $fromaddress = '';
    }

    $usercheck = get_record('user', 'id', $userid);

	// Send email with iCal attachment
	if ($notificationtype & MDL_F2F_ICAL) {
        foreach ($icalattachments as $attachment) {
            if (!email_to_user($user, $fromaddress, $attachment['subject'], $attachment['body'],
                               $attachment['htmlbody'], $attachment['filename'], $attachmentfilename)) {

                return get_string('error:cannotsendconfirmationuser', 'facetoface');
            }
            unlink($CFG->dataroot . '/' . $attachment['filename']);
        }
	}

    // Send plain text email
	if ($notificationtype & MDL_F2F_TEXT) {
	    if (!email_to_user($user, $fromaddress, $postsubject, $posttext, $posthtml)) {
            return get_string('error:cannotsendconfirmationuser', 'facetoface');
	    }
	}

	// Manager notification - mdl_user.msn is where the managers email address is stored
	if (!empty($posttextmgrheading) and !empty($usercheck->msn) and $session->datetimeknown) {
	    $managertext = $posttextmgrheading.$posttext;
        $manager = $user;
        $manager->email = $usercheck->msn;

        // Leave out the ical attachments in the managers notification
        if (!email_to_user($manager, $fromaddress, $postsubject, $managertext, $posthtml)) {
            return get_string('error:cannotsendconfirmationmanager', 'facetoface');
        }
	}

    // Third-party notification
    if (!empty($facetoface->thirdparty) &&
        ($session->datetimeknown || !empty($facetoface->thirdpartywaitlist))) {

        $thirdparty = $user;
        $thirdparty->email = $facetoface->thirdparty;

        // Leave out the ical attachments in the 3rd parties notification
        if (!email_to_user($thirdparty, $fromaddress, $postsubject, $posttext, $posthtml)) {
            return get_string('error:cannotsendconfirmationthirdparty', 'facetoface');
        }
    }
}

/**
 * Send a confirmation email to the user and manager
 *
 * @param class $facetoface record from the facetoface table
 * @param class $session record from the facetoface_sessions table
 * @param integer $userid ID of the recipient of the email
 * @param integer $notificationtype Type of notifications to be sent @see {{MDL_F2F_INVITE}}
 * @returns string Error message (or empty string if successful)
 */
function facetoface_send_confirmation_notice($facetoface, $session, $userid, $notificationtype) {

    $posttextmgrheading = $facetoface->confirmationinstrmngr;
    if ($session->datetimeknown) {
        $postsubject = $facetoface->confirmationsubject;
        $posttext = $facetoface->confirmationmessage;
    } else {
        $postsubject = $facetoface->waitlistedsubject;
        $posttext = $facetoface->waitlistedmessage;
    }

    // Set invite bit
    $notificationtype |= MDL_F2F_INVITE;

    return facetoface_send_notice($postsubject, $posttext, $posttextmgrheading,
                                  $notificationtype, $facetoface, $session, $userid);
}

/**
 * Send a confirmation email to the user and manager regarding the
 * cancellation
 *
 * @param class $facetoface record from the facetoface table
 * @param class $session record from the facetoface_sessions table
 * @param integer $userid ID of the recipient of the email
 * @param integer $notificationtype Type of notifications to be sent @see {{MDL_F2F_INVITE}}
 * @returns string Error message (or empty string if successful)
 */
function facetoface_send_cancellation_notice($facetoface, $session, $userid, $notificationtype) {

    $postsubject = $facetoface->cancellationsubject;
    $posttext = $facetoface->cancellationmessage;
    $posttextmgrheading = $facetoface->cancellationinstrmngr;

    // Set cancellation bit
    $notificationtype |= MDL_F2F_CANCEL;

    return facetoface_send_notice($postsubject, $posttext, $posttextmgrheading,
                                  $notificationtype, $facetoface, $session, $userid);
}

/**
 * Returns true if the user has registered for a session in the given
 * facetoface activity
 *
 * @global class $USER used to get the current userid
 */
function facetoface_check_signup($facetofaceid) {

    global $USER;

    if ($submission = get_record_select('facetoface_submissions', "facetoface=$facetofaceid AND userid=$USER->id")) {

        return true;

    } else {

        return false;
    }

}

/**
 * Return the email address of the user's manager
 *
 * @global class $USER used to get the current userid
 */
function facetoface_get_manageremail() {

    global $USER;

    $manager = get_record('user', 'id', $USER->id);

    $manageremail = $manager->msn;

    if (!empty($manageremail)) {

        return $manageremail;

    } else {

        return false;
    }
}

/**
 * Human-readable version of the format of the manager's email address
 */
function facetoface_get_manageremailformat() {

    $addressformat = get_config(NULL, 'facetoface_manageraddressformat');

    if (!empty($addressformat)) {
        $readableformat = get_config(NULL, 'facetoface_manageraddressformatreadable');
        return get_string('manageremailformat', 'facetoface', $readableformat);
    }

    return '';
}

/**
 * Returns true if the given email address follows the format
 * prescribed by the site administrator
 *
 * @param string $manageremail email address as entered by the user
 */
function facetoface_check_manageremail($manageremail) {

    $addressformat = get_config(NULL, 'facetoface_manageraddressformat');

    if (empty($addressformat) || strpos($manageremail, $addressformat)) {
        return true;
    } else {
        return false;
    }
}

/**
 * Set the user's manager email address by using the MSN field in the
 * user table
 *
 * @global class $USER used to get the current userid
 */
function facetoface_set_manageremail($manageremail) {

    global $USER;
    $user = new object;
    $user->id = $USER->id;
    $user->msn = $manageremail;

    return update_record('user', $user);
}

/**
 * Mark the fact that the user attended the facetoface session by
 * giving that user a grade of 100
 *
 * @param array $data array containing the sessionid under the 's' key
 *                    and every submission ID to mark as attended
 *                    under the 'submissionid_XXXX' keys where XXXX is
 *                    the ID of the signup
 */
function facetoface_take_attendance($data) {
    $sessionid = $data->s;
    $submission = new object;
    $timenow = time();
    foreach ($data as $key => $value) {
        $submissionidcheck = substr($key, 0, 13);
        if ($submissionidcheck == 'submissionid_') {
            $submissionid = substr($key, 13);
            $submission->id = $submissionid;
            $submission->grade = 100;
            $submission->timegraded = $timenow;
            update_record('facetoface_submissions', $submission);
        }
    }
    return true;
}

/**
 * Used by course/lib.php to display a few sessions besides the
 * facetoface activity on the course page
 *
 * @global class $USER used to get the current userid
 */
function facetoface_print_coursemodule_info($coursemodule) {

    global $CFG, $USER;

    $info = NULL;
    $table = '';

    $timenow = time();
    $facetofaceid = $coursemodule->instance;

    if ($facetoface = get_record('facetoface', 'id', $facetofaceid)) {

        $context = get_context_instance(CONTEXT_MODULE, $coursemodule->id);
        if (has_capability('mod/facetoface:view', $context)) {

            if ($submission = get_record_select('facetoface_submissions', "facetoface=$facetofaceid AND userid=$USER->id")) {
            // User has signedup for the instance

                if ($session = facetoface_get_session($submission->sessionid)) {

                    if ($session->datetimeknown) {

                        $sessiondate = '';
                        $sessiontime = '';
                        foreach ($session->sessiondates as $date) {
                            if (!empty($sessiondate)) {
                                $sessiondate .= '<br />';
                            }
                            $sessiondate .= userdate($date->timestart, get_string('strftimedate'));
                            if (!empty($sessiontime)) {
                                $sessiontime .= '<br />';
                            }
                            $sessiontime .= userdate($date->timestart, get_string('strftimetime')).
                                ' - '.userdate($date->timefinish, get_string('strftimetime'));
                        }

                    } else {

                        $sessiondate = get_string('wait-listed', 'facetoface');
                        $sessiontime = get_string('wait-listed', 'facetoface');

                    }

                    $table = '<table border="0" cellpadding="1" cellspacing="0" width="90%">'
                            .'<tr>'
                            .'<td colspan="4"><span style="font-size: 11px; font-weight: bold; line-height: 14px;">'.get_string('bookingstatus', 'facetoface').':</span></td>'
                            .'<td><span style="font-size: 11px; font-weight: bold; line-height: 14px;">'.get_string('options', 'facetoface').':</span></td>'
                            .'</tr>'
                            .'<tr>'
                            .'<td>'.$session->location.'</td>' 
                            .'<td>'.$session->venue.'</td>' 
                            .'<td>'.$sessiondate.'</td>' 
                            .'<td>'.$sessiontime.'</td>'
                            .'<td><table border="0"><tr><td><span style="font-size: 11px; font-weight: bold; line-height: 14px;"><a href="'.$CFG->wwwroot.'/mod/facetoface/signup.php?s='.$session->id.'" alt="'.get_string('moreinfo', 'facetoface').'" title="'.get_string('moreinfo', 'facetoface').'">'.get_string('moreinfo', 'facetoface').'</a></span></td>'
                            .'</tr>'
                            .'<tr>'
                            .'<td><span style="font-size: 11px; font-weight: bold; line-height: 14px;"><a href="'.$CFG->wwwroot.'/mod/facetoface/attendees.php?s='.$session->id.'" alt="'.get_string('seeattendees', 'facetoface').'" title="'.get_string('seeattendees', 'facetoface').'">'.get_string('seeattendees', 'facetoface').'</a></span></td>'
                            .'</tr>'
                            .'<tr>'
                            .'<td><span style="font-size: 11px; font-weight: bold; line-height: 14px;"><a href="'.$CFG->wwwroot.'/mod/facetoface/signup.php?s='.$session->id.'&amp;cancelbooking=1" alt="'.get_string('cancelbooking', 'facetoface').'" title="'.get_string('cancelbooking', 'facetoface').'">'.get_string('cancelbooking', 'facetoface').'</a></span></td>'
                            .'</tr>'
                            .'<tr>'
                            .'<td><span style="font-size: 11px; font-weight: bold; line-height: 14px"><a href="'.$CFG->wwwroot.'/mod/facetoface/view.php?f='.$facetofaceid.'" alt="'.get_string('viewallsessions', 'facetoface').'" title="'.get_string('viewallsessions', 'facetoface').'">'.get_string('viewallsessions', 'facetoface').'</a></span></td>'
                            .'</tr>'
                            .'</table></td></tr>'
                            .'</table>';


                }

            } elseif ($sessions = facetoface_get_sessions($facetofaceid)) {

                if ($facetoface->display == 0) {

                    $table .= '<table border="0" cellpadding="1" cellspacing="0" width="100%">'
                            .'   <tr>'
                            .'       <td colspan="2"><span style="font-size: 11px; font-weight: bold; line-height: 14px;">'.get_string('signupforsession', 'facetoface').':</span></td>'
                            .'   </tr>';
                    $table .= '   <tr>'
                            .'     <td colspan="2"><span style="font-size: 11px; font-weight: bold; line-height: 14px"><a href="'.$CFG->wwwroot.'/mod/facetoface/view.php?f='.$facetofaceid.'" alt="'.get_string('viewallsessions', 'facetoface').'" title="'.get_string('viewallsessions', 'facetoface').'">'.get_string('viewallsessions', 'facetoface').'</a></span></td>'
                            .'   </tr>'
                            .'</table>';

                } else {

                    $table = '<table border="0" cellpadding="1" cellspacing="0" width="100%">'
                            .'   <tr>'
                            .'       <td colspan="2"><span style="font-size: 11px; font-weight: bold; line-height: 14px;">'.get_string('signupforsession', 'facetoface').':</span></td>'
                            .'   </tr>';

                    $i=0;

                    foreach($sessions as $session) {

                        if ($session->datetimeknown && (facetoface_has_session_started($session, $timenow))) {
                            continue;
                         }

                        $signupcount = count_records('facetoface_submissions', 'sessionid', $session->id);
                        if ($signupcount >= $session->capacity) continue;

                        $multiday = '';
                        if ($session->datetimeknown) {

                            if (empty($session->sessiondates)) {
                                $sessiondate = get_string('unknowndate', 'facetoface');
                                $sessiontime = get_string('unknowntime', 'facetoface');
                            } else {
                                $sessiondate = userdate($session->sessiondates[0]->timestart, get_string('strftimedate'));
                                $sessiontime = userdate($session->sessiondates[0]->timestart, get_string('strftimetime')).
                                    ' - '.userdate($session->sessiondates[0]->timefinish, get_string('strftimetime'));
                                if (count($session->sessiondates) > 1) {
                                    $multiday = ' ('.get_string('multiday', 'facetoface').')';
                                }
                            }

                        } else {

                            $sessiondate = get_string('wait-listed', 'facetoface');
                            $sessiontime = "";

                        }

                        if ($i == 0) {
                            $table .= '   <tr>';
                            $i++;
                        } else if ($i++ % 2 == 0) {
                            if ($i > $facetoface->display) {
                                break;
                            }
                            $table .= '   </tr>';
                            $table .= '   <tr>';
                        }
                        $table .= '      <td><span style="font-size: 11px; line-height: 14px;"><a href="'.$CFG->wwwroot.'/mod/facetoface/signup.php?s='.$session->id.'">'.$session->location.', '.$sessiondate.'<br />'.$sessiontime.$multiday.'</a></span></td>';
                    }
                    if ($i++ % 2 == 0) {
                        $table .= '<td><span style="font-size: 11px; line-height: 14px;">&nbsp;</span></td>';
                    }
                    $table .= '   </tr>'
                        .'   <tr>'
                        .'     <td colspan="2"><span style="font-size: 11px; font-weight: bold; line-height: 14px"><a href="'.$CFG->wwwroot.'/mod/facetoface/view.php?f='.$facetofaceid.'" alt="'.get_string('viewallsessions', 'facetoface').'" title="'.get_string('viewallsessions', 'facetoface').'">'.get_string('viewallsessions', 'facetoface').'</a></span></td>'
                        .'   </tr>'
                        .'</table>';
                }

            } else {

                if (has_capability('mod/facetoface:viewemptyactivities', $context)) {

                    $strdimmed = '';

                    if (!$coursemodule->visible) {
                        $strdimmed = ' class="dimmed"';
                    }

                    $table = '<img src="'.$CFG->wwwroot.'/mod/facetoface/icon.gif" class="activityicon" alt="'.get_string('facetoface', 'facetoface').'" /> <a title="'.get_string('facetoface', 'facetoface').'"'.$strdimmed.' href="'.$CFG->wwwroot.'/mod/facetoface/view.php?f='.$facetofaceid.'">'.get_string('facetoface','facetoface').'</a>';

                }

            }

        }

    } 

    return $table;
}

/**
 * Returns the ICAL data for a facetoface meeting.
 *
 * @param integer $method The method, @see {{MDL_F2F_INVITE}}
 * @return string Filename of the attachment in the temp directory
 */
function facetoface_get_ical_attachment($method, $facetoface, $session, $user) {
    global $CFG;

    // First, generate all the VEVENT blocks
    $VEVENTS = '';
    foreach ($session->sessiondates as $date) {
        // Date that this representation of the calendar information was created - 
        // we use the time the session was created
        // http://www.kanzaki.com/docs/ical/dtstamp.html
        $DTSTAMP = facetoface_ical_generate_timestamp($session->timecreated);

        // UIDs should be globally unique
        $urlbits = parse_url($CFG->wwwroot);
        $UID =
            $DTSTAMP .
            '-' . substr(md5($CFG->siteidentifier . $session->id . $date->id), -8) .   // Unique identifier, salted with site identifier
            '@' . $urlbits['host'];                                                    // Hostname for this moodle installation

        $DTSTART = facetoface_ical_generate_timestamp($date->timestart);
        $DTEND   = facetoface_ical_generate_timestamp($date->timefinish);

        // TODO: currently we are not sending updates if the times of the 
        // sesion are changed. This is not ideal!
        $SEQUENCE = ($method & MDL_F2F_CANCEL) ? 1 : 0;

        // TODO: escape these: must wrap at 75 octets and some characters must 
        // be backslash escaped
        $SUMMARY     = facetoface_ical_escape($facetoface->name);
        $DESCRIPTION = facetoface_ical_escape($session->details);

        // NOTE: Newlines are meant to be encoded with the literal sequence 
        // '\n'. But evolution presents a single line text field for location, 
        // and shows the newlines as [0x0A] junk. So we switch it for commas 
        // here. Remember commas need to be escaped too.
        $LOCATION    = str_replace('\n', '\, ', facetoface_ical_escape("{$session->room}\n{$session->venue}\n{$session->location}"));

        $ORGANISEREMAIL = get_config(NULL, 'facetoface_fromaddress');

        $ROLE = 'REQ-PARTICIPANT';
        $CANCELSTATUS = '';
        if ($method & MDL_F2F_CANCEL) {
            $ROLE = 'NON-PARTICIPANT';
            $CANCELSTATUS = "\nSTATUS:CANCELLED";
        }

        $icalmethod = ($method & MDL_F2F_INVITE) ? 'REQUEST' : 'CANCEL';

        // TODO: if the user has input their name in another language, we need 
        // to set the LANGUAGE property parameter here
        $USERNAME = fullname($user);
        $MAILTO   = $user->email;

        // The extra newline at the bottom is so multiple events start on their 
        // own lines. The very last one is trimmed outside the loop
        $VEVENTS .= <<<EOF
BEGIN:VEVENT
UID:{$UID}
DTSTAMP:{$DTSTAMP}
DTSTART:{$DTSTART}
DTEND:{$DTEND}
SEQUENCE:{$SEQUENCE}
SUMMARY:{$SUMMARY}
LOCATION:{$LOCATION}
DESCRIPTION:{$DESCRIPTION}
CLASS:PRIVATE
TRANSP:OPAQUE{$CANCELSTATUS}
ORGANIZER;CN={$ORGANISEREMAIL}:MAILTO:{$ORGANISEREMAIL}
ATTENDEE;CUTYPE=INDIVIDUAL;ROLE={$ROLE};PARTSTAT=NEEDS-ACTION;
 RSVP=FALSE;CN={$USERNAME};LANGUAGE=en:MAILTO:{$MAILTO}
END:VEVENT

EOF;
    }

    $VEVENTS = trim($VEVENTS);

    $template = <<<EOF
BEGIN:VCALENDAR
CALSCALE:GREGORIAN
PRODID:-//Moodle//NONSGML Facetoface//EN
VERSION:2.0
METHOD:{$icalmethod}
BEGIN:VTIMEZONE
TZID:/softwarestudio.org/Tzfile/Pacific/Auckland
X-LIC-LOCATION:Pacific/Auckland
BEGIN:STANDARD
TZNAME:NZST
DTSTART:19700405T020000
RRULE:FREQ=YEARLY;INTERVAL=1;BYDAY=1SU;BYMONTH=4
TZOFFSETFROM:+1300
TZOFFSETTO:+1200
END:STANDARD
BEGIN:DAYLIGHT
TZNAME:NZDT
DTSTART:19700928T030000
RRULE:FREQ=YEARLY;INTERVAL=1;BYDAY=-1SU;BYMONTH=9
TZOFFSETFROM:+1200
TZOFFSETTO:+1300
END:DAYLIGHT
END:VTIMEZONE
{$VEVENTS}
END:VCALENDAR
EOF;

    $tempfilename = md5($template);
    $tempfilepathname = $CFG->dataroot . '/' . $tempfilename;
    file_put_contents($tempfilepathname, $template);
    return $tempfilename;
}

function facetoface_ical_generate_timestamp($timestamp) {
    return gmdate('Ymd', $timestamp) . 'T' . gmdate('His', $timestamp) . 'Z';
}

/**
 * Escapes data of the text datatype in ICAL documents.
 *
 * See RFC2445 or http://www.kanzaki.com/docs/ical/text.html or a more readable definition
 */
function facetoface_ical_escape($text) {
    $text = str_replace(
        array('\\',   "\n", ';',  ','),
        array('\\\\', '\n', '\;', '\,'),
        $text
    );

    // Text should be wordwrapped at 75 octets, and there should be one 
    // whitespace after the newline that does the wrapping
    $text = wordwrap($text, 75, "\n ", true);

    return $text;
}

?>
